static char rcsid[] = "@(#)$Id: screen.c,v 1.22 2002/06/01 12:58:20 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 1.22 $   $State: Exp $
 *
 *  Modified by: Kari Hurtta <hurtta+elm@ozone.FMI.FI>
 ******************************************************************************
 *  The Elm Mail System 
 *
 *			Copyright (c) 1988-1992 USENET Community Trust
 *			Copyright (c) 1986,1987 Dave Taylor
 *****************************************************************************/

/**  screen display routines for ELM program 

**/

#include "headers.h"
#include "me.h"
#include "s_elm.h"

DEBUG_VAR(Debug,__FILE__,"ui");

#define  minimum(a,b)	((a) < (b) ? (a) : (b))

extern char version_buff[];

static int fix_header_page P_((void));

void showscreen()
{

	ClearScreen();

	update_title();

	last_header_page = -1;	 	/* force a redraw regardless */
	show_headers();

	if (mini_menu)
	  show_menu();

	show_last_error();
	
	/* define_softkeys(MAIN); */
}

void update_title()
{
    /** display a new title line, probably due to new mail arriving **/
    
    struct string * folder_string = NULL;
    struct string * buffer = NULL;
    struct string * buffer2 = NULL;

    ClearLine(1);
    ClearLine(2);

    if (current_folder) {
	int l1,l2,l;
	if (*hostname && menu_display_host) 
	    folder_string = format_string(FRM("%s:%S"), hostname, 
					  current_folder->cur_folder_disp);
	else
	    folder_string = dup_string(current_folder->cur_folder_disp);
	

	if (selected)
	    buffer = format_string(CATGETS(elm_msg_cat, ElmSet, 
					   ElmShownWithSelect,
					   "%s is '%S' with %d shown out of %d"),
				   folder_type(current_folder),
				   folder_string, selected, message_count);
	else if (message_count == 1)
	    buffer = format_string(CATGETS(elm_msg_cat, ElmSet, 
					   ElmShownNoSelect,
					   "%s is '%S' with 1 message"),
				   folder_type(current_folder),
				   folder_string);
	else
	    buffer = format_string(CATGETS(elm_msg_cat, ElmSet, 
					   ElmShownNoSelectPlural,
					   "%s is '%S' with %d messages"),
				   folder_type(current_folder),
				   folder_string, message_count);
    
	free_string(&folder_string);

	l1 = string_len(buffer);

	buffer2 = format_string(FRM("[ELM %s]"),version_buff);
	l2 = string_len(buffer2);
	l = l1 + l2 + 1; /* Assumed */

	if (l > elm_COLUMNS) {
	    if (l2 < elm_COLUMNS)
		PutLineX(2,(elm_COLUMNS - l2)/2,FRM("%S"),buffer2);
	    if (l1 > elm_COLUMNS) 
		PutLineX(1,1,FRM("%S"),buffer);
	    else
		PutLineX(1,(elm_COLUMNS - l1)/2,FRM("%S"),buffer);
	} else {
	    PutLineX(1,(elm_COLUMNS - l)/2,FRM("%S %S"),buffer,buffer2);
	}
	
	free_string(&buffer);
	free_string(&buffer2);
    }    
}

void show_menu()
{
    /** write main system menu... **/

    if (user_level == 0) {	/* a rank beginner.  Give less options  */
	  print_format_center(elm_LINES-7, 
			      CATGETS(elm_msg_cat, ElmSet, 
				      ElmLevel0MenuLine1,
				      "You can use any of the following commands by pressing the first character;"));
         print_format_center(elm_LINES-6, 
			     CATGETS(elm_msg_cat, ElmSet, 
				     ElmLevel0MenuLine2,
				     "d)elete or u)ndelete mail,  m)ail a message,  r)eply or f)orward mail,  q)uit"));
	 print_format_center(elm_LINES-5, 
			     CATGETS(elm_msg_cat, ElmSet, 
				     ElmLevel0MenuLine3,
				     "To read a message, press <return>.  j = move down, k = move up, ? = help"));
    } else {
	print_format_center(elm_LINES-7, 
			    CATGETS(elm_msg_cat, ElmSet, 
				    ElmLevel1MenuLine1,
				    "|=pipe, !=shell, ?=help, <n>=set current to n, /=search pattern"));
        print_format_center(elm_LINES-6, 
			    CATGETS(elm_msg_cat, ElmSet, 
				    ElmLevel1MenuLine2,
				    "a)lias, C)opy, c)hange folder, d)elete, e)dit, f)orward, g)roup reply, m)ail,"));
	print_format_center(elm_LINES-5, 
			    CATGETS(elm_msg_cat, ElmSet, 
				    ElmLevel1MenuLine3,
				    "n)ext, o)ptions, p)rint, q)uit, r)eply, s)ave, t)ag, u)ndelete, or e(x)it"));
    }
}

static void print_header_line P_((struct header_rec *entry, int message_number,
				  int selected, struct string *from,
				  int really_to));
static void print_header_line(entry,message_number,selected,from,really_to)
     struct header_rec *entry;
     int message_number; 
     int selected;
     struct string *from;
     int really_to;
{
    /** Print in buffer the message header ... entry is the current
	message entry, 'from' is a modified (displayable) from line, 
	'highlight' is either TRUE or FALSE, and 'message_number'
	is the number of the message.
    **/
	
    int who_width = 18, subj_width = 0;
    int header_without_subj_width;

    struct string * buffer = NULL;
    struct string * buffer1 = NULL;
    int X;

    /* truncate 'from' to 18 characters -
     * this includes the leading "To" if really_to is true.
     */

    if (*entry->time_menu == '\0')
	make_menu_date(entry); 

    /* show "To " in a way that it can never be truncated. */
    if (really_to) {
	who_width -= 3;
    }

    if (selected && has_highlighting && ! arrow_cursor) 
	StartInverse();

    if (entry->lines < 0) 
	buffer = format_string(FRM("%.2s%.2s%c%-3d %.10s %.3s%-*.*S (?)    "),
			       (selected && arrow_cursor)? "->" : "  ",
			       show_message_status(entry),
			       (entry->status & TAGGED?  '+' : ' '),
			       message_number,
			       entry->time_menu,
			       (really_to ? "To " : ""),
			       /* give max and min width parameters for 'from' */
			       who_width,
			       who_width,
			       from
			       );
    else
	buffer = format_string(FRM("%.2s%.2s%c%-3d %.10s %.3s%-*.*S (%d) %s"),
			       (selected && arrow_cursor)? "->" : "  ",
			       show_message_status(entry),
			       (entry->status & TAGGED?  '+' : ' '),
			       message_number,
			       entry->time_menu,
			       (really_to ? "To " : ""),
			       /* give max and min width parameters for 'from' */
			       who_width,
			       who_width,
			       from,
			       
			       entry->lines, 
			       (entry->lines / 1000   > 0? ""   :	/* spacing the  */
				entry->lines / 100   > 0? " "  :	/* same for the */
				entry->lines / 10  > 0? "  " :	/* lines in ()  */
				"   ")      /*   [wierd]    */
			       );
    
    X = 0;
    /* clip_from_string updates X */
    buffer1 = curses_printable_clip(buffer,&X,elm_COLUMNS,
				    &header_without_subj_width,
				    elm_COLUMNS);

    free_string(&buffer);


    /* Set the subject display width.
     * If it is too long, truncate it to fit.
     * If it is highlighted but not with the arrow  cursor,
     * expand it to fit so that the reverse video bar extends
     * aesthetically the full length of the line.
     */
    if ((selected && !arrow_cursor)
	|| (elm_COLUMNS-header_without_subj_width 
	    < (subj_width = string_len(entry->subject))))
	subj_width = elm_COLUMNS-header_without_subj_width;

    Write_to_screen(FRM("%S"),buffer1);
    free_string(&buffer1);

    /* complete line with subject. */
    if (subj_width > 0) {
	int was_len;
	struct string * buffer2 = NULL;

	X = 0;
	/* clip_from_string updates X */
	buffer2 = curses_printable_clip(entry->subject,&X,subj_width,
					&was_len,subj_width);

	Write_to_screen(FRM("%S"),buffer2);
	free_string(&buffer2);

	while (was_len < subj_width) {
	    Writechar(' ');
	    was_len++;
	}
    }
    
    if (selected && has_highlighting && ! arrow_cursor) 
	EndInverse();
    NewLine();
}

static void print_alias_line P_((struct alias_rec *entry,
				 int message_number, int selected));
static void print_alias_line(entry,message_number,selected)
     struct alias_rec *entry;
     int message_number; 
     int selected;
{
    struct string * buffer = build_alias_line(entry,message_number,selected);
    
    if (selected && has_highlighting && ! arrow_cursor) {
	StartInverse();
	Write_to_screen(FRM("%S\n\r"), buffer);	/* avoid '%' probs */
	EndInverse();
    } else
	Write_to_screen(FRM("%S\n\r"), buffer);	/* avoid '%' probs */
    
    free_string(&buffer);
}

int show_headers()
{
	/** Display page of headers (10) if present.  First check to 
	    ensure that header_page is in bounds, fixing silently if not.
	    If out of bounds, return zero, else return non-zero 
	    Modified to only show headers that are "visible" to ze human
	    person using ze program, eh?
	**/

	register int this_msg = 0, line = 4, last = 0, last_line, 
		     displayed = 0, using_to;

	if (fix_header_page())
	    return(FALSE);

	if (selected) {
	  if ((header_page*headers_per_page) > selected)
	    return(FALSE); 	/* too far! too far! */

	  this_msg = visible_to_index(header_page * headers_per_page + 1);
	  displayed = header_page * headers_per_page;

	  last = displayed+headers_per_page;

	}
	else {
	  if (header_page == last_header_page) 	/* nothing to do! */
	    return(FALSE);

	  /** compute last header to display **/
  
	  this_msg = header_page * headers_per_page;
	  last = this_msg + (headers_per_page - 1);
	}

	if (last >= message_count) last = message_count-1;

	/** Okay, now let's show the header page! **/

	ClearLine(line);	/* Clear the top line... */

	MoveCursor(line, 0);	/* and move back to the top of the page... */

	while ((selected && displayed < last) || this_msg <= last) {
	    
#if DEBUG
	    if (Debug.active > 10 && !inalias) {
		struct addr_item * p;

		for (p =  headers[this_msg]->from; p && p->addr; p++) {
		    DPRINT(Debug,15,(&Debug, 
				     "-- %d:from[%d] addr='%s' comment='%S' fullname='%S'\n",
				     this_msg,p - headers[this_msg]->from,
				     p->addr,
				     p->comment,
				     p->fullname));		    
		}

		for (p =  headers[this_msg]->to; p && p->addr; p++) {
		    DPRINT(Debug,15,(&Debug, 
				     "-- %d:  to[%d] addr='%s' comment='%S' fullname='%S'\n",
				     this_msg,p - headers[this_msg]->to,
				     p->addr,
				     p->comment,
				     p->fullname));		    
		}

		for (p =  headers[this_msg]->cc; p && p->addr; p++) {
		    DPRINT(Debug,15,(&Debug, 
				     "-- %d:  cc[%d] addr='%s' comment='%S' fullname='%S'\n",
				     this_msg,p - headers[this_msg]->cc,
				     p->addr,
				     p->comment,
				     p->fullname));		    
		}
	    }
#endif

	  if (inalias) {
	    if (this_msg == current-1)
		print_alias_line(aliases[this_msg], this_msg+1,TRUE);
	    else
		print_alias_line(aliases[this_msg], this_msg+1,FALSE);
	  }
	  else {
	      struct string * newfrom = NULL;
	      using_to = DisplayAddress(headers[this_msg], &newfrom);
	      
	      if (this_msg == current-1) 
		  print_header_line(headers[this_msg], this_msg+1,
				    TRUE, newfrom, using_to);
	      else
		  print_header_line(headers[this_msg], this_msg+1, 
					  FALSE, newfrom, using_to);
	      free_string(&newfrom);
	  }

	  if (selected) 
	    displayed++;

	  CleartoEOLN();
	  line++;		/* for clearing up in a sec... */

	  if (selected) {
	      if ((this_msg = next_message(this_msg, FALSE)) < 0)
		  break;	/* GET OUTTA HERE! */
	      
	      /* the preceeding looks gross because we're using an INDEX
		 variable to pretend to be a "current" counter, and the 
		 current counter is always 1 greater than the actual 
		 index.  Does that make sense??
	      */
	  }
	  else
	      this_msg++;			/* even dumber...  */

	}

	/* clear unused lines */

	if (mini_menu)
	  last_line = elm_LINES-8;
	else
	  last_line = elm_LINES-4;

	while (line < last_line) {
	  CleartoEOLN();
	  NewLine();
	  line++;
	}

	display_central_message();

	last_current = current;
	last_header_page = header_page;

	return(TRUE);
}

void show_current()
{
    /** Show the new header, with all the usual checks **/
    
    int first = 0, last = 0, last_line, new_line, using_to;

    (void) fix_header_page();	/* Who cares what it does? ;-) */

    /** compute the first and last header on this page **/
    first = header_page * headers_per_page + 1;
    last  = first + (headers_per_page - 1);

    /* if not a full page adjust last to be the real last */
    if (selected && last > selected)
	last = selected;
    if (!selected && last > message_count) 
	last = message_count;

    /** okay, now let's show the pointers... **/

    /** have we changed??? **/
    if (current == last_current) 
	return;

    if (selected) {
	last_line = ((compute_visible(last_current)-1) %
		     headers_per_page)+4;
	new_line  = ((compute_visible(current)-1) % headers_per_page)+4;
    } else {
	last_line = ((last_current-1) % headers_per_page)+4;
	new_line  = ((current-1) % headers_per_page)+4;
    }
    
    if (has_highlighting && ! arrow_cursor) {

	/* clear last current if it's in proper range */
	if (last_current > 0		/* not a dummy value */
	    && compute_visible(last_current) <= last
	    && compute_visible(last_current) >= first) {
	    
	    DPRINT(Debug,5,(&Debug,  
			    "\nlast_current = %d ... clearing [1] before we add [2]\n", 
			    last_current));
	    DPRINT(Debug,5,(&Debug, 
			    "first = %d, and last = %d\n\n",
			    first, last));

	    ClearLine(last_line);	    
	    MoveCursor(last_line,0);

	    if (inalias)
		print_alias_line(aliases[last_current-1],
					      last_current, FALSE);
	    else {
		struct string * newfrom = NULL;       
		using_to = DisplayAddress(headers[last_current-1], &newfrom);
		print_header_line(headers[last_current-1], 
				  last_current, FALSE, newfrom, 
				  using_to);
		free_string(&newfrom);
	    }
	}


	MoveCursor(new_line, 0);
	
	if (inalias)
	    print_alias_line(aliases[current-1], current,TRUE);
	else {
	    struct string * newfrom = NULL;
	    using_to = DisplayAddress(headers[current-1], &newfrom);
	    print_header_line(headers[current-1],  current,
			      TRUE, newfrom, using_to);
	    free_string(&newfrom);
	}
	
    } else {
	if (on_page(last_current-1)) 
	    PutLine0(last_line,0,"  ");	/* remove old pointer... */
	if (on_page(current-1))
	    PutLine0(new_line, 0,"->");
    }
	
    last_current = current;
}

int fix_header_page()
{
	/** this routine will check and ensure that the current header
	    page being displayed contains messages!  It will silently
	    fix 'header-page' if wrong.  Returns TRUE if changed.  **/

	int last_page, old_header;

	old_header = header_page;

	last_page = (int) ((message_count-1) / headers_per_page);
 
	if (header_page > last_page) 
	  header_page = last_page;
	else if (header_page < 0) 
          header_page = 0;

	return(old_header != header_page);
}

int on_page(message)
     int message;
{
	/** Returns true iff the specified message is on the displayed page. **/

	if (selected) message = compute_visible(message);

	if (message >= header_page * headers_per_page)
	  if (message < ((header_page+1) * headers_per_page))
	    return(TRUE);

	return(FALSE);
}

/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 * End:
 */




